import smtplib

from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart

from scraping_tools.logger import logger


def notify(sender_address,
           sender_password,
           message_subject,
           message_recipients,
           message_text_content,
           message_html_content):
    """Send an email to recipients.

    :param str sender_address: sender email address.
    :param str sender_password: sender email password.
    :param str message_subject: message subject.
    :param list message_recipients: recipients mail addresses
    :param str message_text_content: message body - textual format.
    :param unicode message_html_content: message body - HTML format.
    """
    logger.debug("Sending an email message %r to recipients %r", message_subject, message_recipients)
    message_recipients = ','.join(message_recipients)

    # Define a message object
    msg = MIMEMultipart('alternative')
    msg['From'] = sender_address
    msg['To'] = message_recipients
    msg['Subject'] = message_subject

    msg.attach(MIMEText(message_text_content, 'plain'))
    msg.attach(MIMEText(message_html_content, 'html', _charset="UTF-8"))

    # Login to the SMTP server and send the message
    server = smtplib.SMTP('smtp.gmail.com', 587)
    server.starttls()
    server.login(sender_address, sender_password)
    message_text_content = msg.as_string()
    server.sendmail(sender_address, message_recipients, message_text_content)
    server.quit()



import logging
import coloredlogs

DEFAULT_LEVEL = 'DEBUG'
DEFAULT_FORMAT = '%(asctime)s %(processName)s [%(levelname)s] %(message)s'
DEFAULT_DATE_FORMAT = '%d-%m-%y %H:%M:%S'


def init_logger(logger_name, level=DEFAULT_LEVEL, fmt=DEFAULT_FORMAT, date_fmt=DEFAULT_DATE_FORMAT):
    """Initialize logger.

    - Create a logger instance
    - Initialize a colored log instance

    :param str logger_name: logger name.
    :param str level: log level (e.g 'DEBUG')
    :param str fmt: log messages format.
    :param str date_fmt: log messages date format.

    :return: logging.Logger instance.
    """
    new_logger = logging.getLogger(logger_name)
    new_logger.setLevel(logging.DEBUG)
    coloredlogs.install(level=level, logger=new_logger, fmt=fmt, datefmt=date_fmt)
    return new_logger


logger = init_logger(logger_name='scraping-tools')



import io
import json


def str_to_utf8(s):
    """Return utf-8 encoded string."""
    return unicode(s).encode("utf-8")


def list_to_utf8(l):
    """Return utf-8 encoded list."""
    return [str_to_utf8(s) for s in l]


def dict_to_utf8(d):
    """Return utf-8 encoded dict."""
    return {str_to_utf8(k): str_to_utf8(v) for k, v in d.items()}


def save_utf8_json(data, target_path):
    """Write the merged data into the target file."""
    with io.open(target_path, 'wt', encoding='utf8') as target_file:
        target_file.write(unicode(json.dumps(data, ensure_ascii=False, indent=4, sort_keys=True)))



import munch
import varyaml


def get_config(config_path):
    """Reads the given yaml config file and return a munch object.

    :param str config_path: config file path.
    :return munch.Munch: config object.
    """
    return munch.Munch.fromDict(varyaml.load(open(config_path, 'r')))



import contextlib
from multiprocessing import Pool

from scraping_tools.logger import logger


@contextlib.contextmanager
def safe_pool(process_number):
    """Process pool context manager.

    - Create a process pool with the given process number
    - Yields the pool
    - Quit & cleanup when done.

    :param num process_number: the required number of processes.
    """
    logger.debug("Starting a safe pool of %s processes", process_number)
    process_pool = Pool(processes=process_number)
    try:
        yield process_pool
    except:
        logger.exception("Pool encountered an unexpected error")
    finally:
        logger.debug("Cleaning up pool")
        process_pool.close()
        process_pool.join()



import random

from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import Select

from scraping_tools.logger import logger
from scraping_tools.driver import get_elem_by_css_selector, get_elem_by_class

PROXY_URL = "https://www.proxysite.com/"
SEARCH_CSS_SELECTOR = '#url-form-wrap > form > div.row > input[type="text"]'


def proxy(driver, url, timeout_seconds=30):
    """Use a proxy site to access required URLs.

    :param selenium.webdriver.remote.WebDriver driver: selenium webdriver instance.
    :param str url: the URL address to proxy to.
    :param number timeout_seconds: timeout in seconds.
    """
    logger.debug("Using proxy to get URL %r", url)
    driver.get(PROXY_URL)

    logger.debug("Getting servers drop down element")
    servers_drop_down = Select(
        get_elem_by_class(driver=driver,
                          class_name='server-option',
                          timeout_seconds=timeout_seconds)
    )
    selected_server = random.choice([option.text for option in servers_drop_down.options])

    logger.debug("Selecting a random server %s", selected_server)
    servers_drop_down.select_by_visible_text(selected_server)

    logger.debug("Getting search text box element")
    search_tb_elem = get_elem_by_css_selector(
        driver=driver,
        css_selector=SEARCH_CSS_SELECTOR,
        timeout_seconds=timeout_seconds
    )

    logger.debug("Clearing the search text box")
    search_tb_elem.clear()

    logger.debug("Sending url %r to text box", url)
    search_tb_elem.send_keys(url)

    logger.debug("Sending return key to text box")
    search_tb_elem.send_keys(Keys.RETURN)



import contextlib

from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as ec
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities

from scraping_tools.logger import logger


@contextlib.contextmanager
def driver_session():
    """Web-driver session context manager.

     - Create a web driver.
     - Yields the driver
     - Quit & cleanup when done.
    """
    logger.debug("Creating a web-driver session")

    driver = webdriver.Remote(
        command_executor='http://phantomjs:8910',
        desired_capabilities=DesiredCapabilities.PHANTOMJS
    )

    driver.maximize_window()
    driver.set_window_position(0, 0)
    driver.implicitly_wait(30)

    try:
        yield driver
    except:
        logger.exception("Session encountered an unexpected error")
        raise
    finally:
        logger.debug("Session quitting web-driver")
        driver.close()
        driver.quit()


def get_elem_by_xpath(driver, xpath, timeout_seconds=30):
    """Return an element by xpath."""
    return wait_for_element(driver, xpath, By.XPATH, timeout=timeout_seconds)


def get_elem_by_class(driver, class_name, timeout_seconds=30):
    """Return an element by class name."""
    return wait_for_element(driver, class_name, By.CLASS_NAME, timeout=timeout_seconds)


def get_elem_by_css_selector(driver, css_selector, timeout_seconds=30):
    """Return an element by css selector."""
    return wait_for_element(driver, css_selector, By.CSS_SELECTOR, timeout=timeout_seconds)


def wait_for_element(driver, identifier, by=By.ID, timeout=30):
    """Return an element once it's located."""
    return WebDriverWait(driver, timeout).until(ec.presence_of_element_located((by, identifier)))


def wait_for_clickable_element(driver, identifier, by=By.ID, timeout=30):
    """Return an element once it's located and clickable."""
    return WebDriverWait(driver, timeout).until(ec.element_to_be_clickable((by, identifier)))
